// This sketch is adapted to Balancing robot No4 with distance sensor

#include <I2Cdev.h>
#include <MPU6050.h>  
#include <SPI.h>
#include <Wire.h>

MPU6050 accelgyro; // SCI A5, SDA A4,
MPU6050 initialize;
int16_t ax, ay, az;
int16_t gx, gy, gz;

#define Gyr_offset 0.0 //The offset of the gyro
#define Gyr_Gain 16.348
#define RMotor_offset    13  // The offset of the RMotor
#define LMotor_offset    7 // The offset of the LMotor
#define Driving_Angle  45
#define Limit_distance 10 // Limit of the distance
#define Max_distance 20 // Just stand up over 20cm 
#define pi 3.1415692

float kp, ki, kd;
float Angle_Raw, Angle_Filtered, omega, Angle_offset = 0;
float Turn_Speed = 0;
float Run_Speed = 0;
float LOutput,ROutput = 0;
float Input, Output = 0;
float errSum, dErr, error, lastErr = 0;

unsigned long preTime = 0;
unsigned long lastTime = 0;

int ENA=3;
int TN1=2;
int TN2=4;
int ENB=5;
int TN3=6;
int TN4=7;
int distance;

void setup() {
  Wire.begin();
  Serial.begin(9600);
  accelgyro.initialize();
  initialize.setFullScaleGyroRange(MPU6050_GYRO_FS_2000);
  pinMode(0, OUTPUT);
  pinMode(TN1,OUTPUT);
  pinMode(TN2,OUTPUT);
  pinMode(TN3,OUTPUT);
  pinMode(TN4,OUTPUT);
  pinMode(ENA,OUTPUT);
  pinMode(ENB,OUTPUT);
 }
 
 void loop()
 {
  Filter();
  //Setting of angle offset
  double value = analogRead(A3); // read the data from potentiometer
  Angle_offset = -3.5 + (value*5.0/1024) ;
  
  if (abs(Angle_Filtered) < Driving_Angle) {
    PID();
    PWMControl();
    Distance_sensor();
  if ( distance >= Max_distance) {
    Run_Speed = 0;
  }
  if (distance >= Limit_distance) {
     Run_Speed += 0.8;
  }
 else if (distance < Limit_distance)  {
    Run_Speed += -0.8;
  }
  }
 else  {  // Stop the wheels
    analogWrite(ENA, 0); 
    analogWrite(ENB, 0);  
     for(int i = 0; i < 100; i++)  // Keep reading the gesture after falling down
    {
      Filter();
    }
 if(abs(Angle_Filtered) < Driving_Angle)  // Empty datas and restart the robot automaticaly
    {
      for(int i = 0; i <= 500; i++)  // Reset the robot and delay 2 seconds
    {
        omega = Angle_Raw = Angle_Filtered = 0;
        Filter();
        Output = error = errSum = dErr = Run_Speed = Turn_Speed = 0;
        PID();
      }
    }
  }
 }

void Filter()
{
  // Raw datas from MPU6050
  accelgyro.getMotion6(&ax, &ay, &az, &gx, &gy, &gz);
  Angle_Raw = (atan2(ay, az) * 180 / pi + Angle_offset );
  omega =  gx / Gyr_Gain + Gyr_offset;
  
  // Filters datas to get the real gesture
  unsigned long now = millis();
  float dt = (now - preTime) / 1000.00;
  preTime = now;
  float K = 0.8;
  float A = K / (K + dt);
  Angle_Filtered = A * (Angle_Filtered + omega * dt) + (1 - A) * Angle_Raw ;
 }

void PID()
{ 
  kp =12;
  ki = 0.15;
  kd = 200;
  unsigned long now = millis();
  float timeChange = (now - lastTime) ;
  lastTime = now;
  error = Angle_Filtered;  // Proportion
  errSum += error * timeChange;  // Integration
  dErr = (error - lastErr) / timeChange;  // Differentiation
  Output = kp * error + ki * errSum + kd * dErr;
  lastErr = error;
  LOutput = Output + Turn_Speed + Run_Speed;
  ROutput = Output - Turn_Speed + Run_Speed;
}

void PWMControl()
{
  if(LOutput > 0)
  {
    digitalWrite(TN1, HIGH);
    digitalWrite(TN2, LOW);
  }
  else if(LOutput < 0)
  {
    digitalWrite(TN1, LOW);
    digitalWrite(TN2, HIGH);
  }
  else
  {
    digitalWrite(ENA, 0);
  }
  if(ROutput > 0)
  {
    digitalWrite(TN3, HIGH);
    digitalWrite(TN4, LOW);
  }
  else if(ROutput < 0)
  {   
    digitalWrite(TN3, LOW);
    digitalWrite(TN4, HIGH);
  }
  else
  {
    digitalWrite(ENB, 0);
    digitalWrite(ENA, 0);
  }
    analogWrite(ENA, (abs(LOutput)+LMotor_offset ));
    analogWrite(ENB, (abs(ROutput)+RMotor_offset ));
}

void Distance_sensor()
  {
  int vout;
  vout = analogRead(0);
  distance = (6762/(vout-9))-4;
  }
   
